/*
 * Copyright (C) 1998 Jonas Borgstrm <jonas_b@bitsmart.com>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */ 

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>

#if USE_PVM
   #include <pvm3.h>
   #include "pvm.h"
#endif

#include "zipcracker.h"
#include "crc_table.h"

unsigned char zip_sig[] = {0x50, 0x4b, 0x03, 0x04}; /* Sign of zip header */
char header[12];             /* Encrypted header */
unsigned long crc;           /* CRC  to find */

/*****************************************************************************/
/*    Sends error message and error code to the master */
/*****************************************************************************/
void error_msg(char *errormsg, int error_code) {
#if USE_PVM
	pvm_initsend(PvmDataDefault);
	pvm_pkstr(errormsg);
	pvm_pkint(&error_code, 1, 1);
	pvm_send(ptid, PVM_ERROR);
	pvm_exit();
#else
	printf("Error: (%d) %s\n", error_code, errormsg);
#endif
	exit(error_code);
}
/*****************************************************************************/
/*    Parse zipfile to get CRC value */
/*****************************************************************************/
void parse_zipfile() {
	FILE *zipfd;
	unsigned char buffer[1024];
	int i, end;
	
	if((zipfd = fopen(zipname, "r")) == NULL) {
		error_msg("Can't open zipfile", RETURN_CANT_OPEN_ZIPFILE);
	}
	
	i = 0;
	end=0;
	
	while(!end) {
		if(fread(buffer, 1, 1, zipfd)) {
			if(buffer[0] == zip_sig[i])
			    i = i + 1;
			else
			    i = 0;
			
			if (i == 4) {
				if(fseek(zipfd, 26, SEEK_CUR) != 0) {
					error_msg("fseek fail", RETURN_IO);
				}
				if(fread(buffer, 1,strlen(targetname), 
					 zipfd)) {
					if(!memcmp(targetname, buffer,
						   strlen(targetname))) 
					    end=1;
					else
					    i=0;
				}
				else {
					error_msg("read failed", RETURN_IO);
				}
			}
		}
		else
		    end = 2;
	}
	
	if(end == 2) {
		error_msg("Couldn't find target in zipfile", RETURN_TARGET_NOT_FOUND);
	}
	
        fread(header, 1, 12, zipfd);
        fseek(zipfd, -(strlen(targetname) + 16 + 12), SEEK_CUR);
        fread(buffer, 1, 4, zipfd);
	
	crc = buffer[3];
	fclose(zipfd);

	buffer[12]=0;
}
/*****************************************************************************/
/*    Tries to decrypt header and checks if CRC match, if so return 1 */
/*****************************************************************************/
int decript_head(unsigned char *buffer, unsigned long *key) {
	int i;
	char c;
	
	for(i = 0; i<12;i++) {
		c = buffer[i] ^ decrypt_byte(key);
		update_keys(key,c);
		buffer[i] = c;
	}
	
	return buffer[11] == crc;
}
/*****************************************************************************/
/*    update_keys() */
/*****************************************************************************/
void update_keys(unsigned long *key, char val) {
	
	key[0] = crc32(key[0], val);
	key[1] = (key[1] + (key[0] & 0xff)) * 134775813L + 1;
	key[2] = crc32(key[2],(key[1] >> 24));
}
/*****************************************************************************/
/*    decrypt_byte */
/*****************************************************************************/
unsigned char decrypt_byte(unsigned long *key) {
	unsigned short temp;
	
	temp = key[2] | 2;
	return((temp *( temp^1)) >> 8);
}
/*****************************************************************************/
/*    CRC rutine */
/*****************************************************************************/
unsigned long crc32(unsigned long oldcrc, char newchar) {
	register unsigned long Crc = oldcrc;
	
        Crc = ((Crc >> 8) & 0x00ffffffL) ^ 
	    crc_table[(Crc ^ newchar) & 0xffL];
	
	return Crc;            /* return a 1's complement */
}
/****************************************************************************/
/*    Main cracking procedure */
/****************************************************************************/
void crack(char *guess, char *end, unsigned long *rate) {
	int i, error;
	unsigned long count = 0;
	time_t sec;
	unsigned long key[3];
	unsigned char hackbuff[19], command[1024];
	char realpw[PW_LEN];
	
	/* Reset variables */
	time(&sec);
	*rate = 0;
	
	while(1) {
		format_passwd(guess, realpw);
		/* Checking speed */
		count++;
		if(time(NULL) > sec) {
			*rate = count; /* Save value */
			count = 0;
			time(&sec);
		}		
		memcpy(hackbuff, header,12);
		key[0] = 305419896L;
		key[1] = 591751049L;
		key[2] = 878082192L;
		
		for(i=0; i<strlen(realpw);i++)
		    update_keys(key,realpw[i]);

		/* Is password a posible password */
		if(decript_head(hackbuff,key)) {

			/* Run unzip to check if it's the correct password */
			sprintf(command, "unzip -t -P %s %s %s > /dev/null 2> /dev/null", realpw,
				zipname, targetname);
			error = system(command) / 0xff;
			switch(error) {
			case 0: /* Correct password */
#if USE_PVM
				pvm_initsend(PvmDataDefault);
				pvm_pkstr(guess);
				pvm_send(ptid, PVM_FOUND);
#else
				/* Test again */
				if(system(command) / 0xff == 0) {

					printf("\7-- Found correct password '%s' --\n", realpw);
					ctrl_c(0);
				}
				else 
				    ctrl_c(SIGINT);
#endif
				return;
			case 2: /* Incorrect password, Just continue*/
				break;
			case 80: /* Ctrl-C */
#ifndef USE_PVM
				ctrl_c(SIGINT);
#endif				
				break;
			case 81: /* Unsupported compression method or decryption */
				error_msg("UNZIP reported: Unsupported compression method or decryption", RETURN_UNZIP_ERROR_81);
				break;
			default:
				error_msg("Unknown unzip return code", error);
			};
			
		}		
		/* Finished with our block? */
		if(strcmp(guess, end) == 0)
		    return;
		
		i = 0;
		guess[i] = *(strchr(string, guess[i]) + 1);
		while(guess[i] == 0) {
			if(guess[i+1] == 0) {
				guess[i] = string[0];
 				guess[i + 1] = string[0];
				guess[++i+1] = 0;
				break;
			}
			else {
				guess[i] = string[0];
				i++;
				guess[i] = *(strchr(string, guess[i]) + 1);
			}
		}
	}
}
